
// ------------------------------------------------------------------
// "TokenBuilder" V1.0 - 6/2008
//  miko@mikoweb.de / http://www.mikoweb.eu
//  A small project to learn 3DGS A7 
// ------------------------------------------------------------------

// For additional bindings, see "tokebuilder.wdl"

// Defines max number of tokens and token types
#define MAX_TOKENS 200
#define NONE -1
#define TOKEN_1X1 1
#define TOKEN_2X1 2
#define TOKEN_2X2 3
#define TOKEN_2X8 4

// The tokens' colors
#define COLOR_RED    1
#define COLOR_GREEN  2
#define COLOR_BLUE   3
#define COLOR_YELLOW 4
#define COLOR_WHITE  5
#define COLOR_BLACK  6
#define COLOR_GREY   7

// Renaming some skill values of token entities
#define bMovemodus   skill51
#define iObjectIndex skill52
#define iObjectColor skill53
#define iObjectType  skill54

// ------------------------------------------------------------------
// Forward declaration of all following functions, for convenience
// ------------------------------------------------------------------
function round(x);
function floor(x);
function matBump_startup (void);
function initializeData_startup(void);
function moveCamera_startup(void);
function moveMouse_startup(void);
function mouseoverHelp_startup(void);
function errormsgPan_startup(void);
function okmsgPan_startup(void);
function showErrorMessage(STRING* errortext);
function showOkMessage(STRING* oktext);
function deleteContent(void);
function resetCameraposition(void);
function setTokenColor(index,color);
function onshowHelp(void);
function setmouseoverHelp(x,y,STRING* message);
function onMouseOverGui(button_number,PANEL* panel);
function onclickResetButton(button_number,PANEL* panel);
function onclickSaveButton(button_number,PANEL* panel);
function onclickLoadButton(button_number,PANEL* panel);
function onclickTokenButton(button_number,PANEL* panel);
function onclickColorButton(button_number,PANEL* panel);
function onclickCreateButton(button_number,PANEL* panel);
function onclickDeleteButton(button_number,PANEL* panel);
function onclickHelpButton(button_number,PANEL* panel);
function onclickScreenshotButton(button_number,PANEL* panel);
function onclickQuitButton(button_number,PANEL* panel);
function updateMouseCursor(void);
function calculateRotatedTracepoints(var iIndex);
function violateBoundaryCondition(var iIndex);
function clickGlobal(void);
function clickToken(void);
function moveToken(void);
function rotateTokenLeft(void);
function rotateTokenRight(void);
function deleteActiveModel(void);
function loadModel(void);
function saveScreenshot(void);

// ------------------------------------------------------------------
// Bitmaps, used later on in panels and other
// ------------------------------------------------------------------ 
BMAP* bmRed    = "bitmaps/Token_red.bmp";
BMAP* bmGreen  = "bitmaps/Token_green.bmp";
BMAP* bmBlue   = "bitmaps/Token_blue.bmp";
BMAP* bmYellow = "bitmaps/Token_yellow.bmp";
BMAP* bmBlack  = "bitmaps/Token_black.bmp";
BMAP* bmWhite  = "bitmaps/Token_white.bmp";
BMAP* bmGrey   = "bitmaps/Token_grey.bmp";

BMAP* bmColRed1    = "bitmaps/button_col_red1.bmp";
BMAP* bmColRed2    = "bitmaps/button_col_red2.bmp";
BMAP* bmColGreen1  = "bitmaps/button_col_green1.bmp";
BMAP* bmColGreen2  = "bitmaps/button_col_green2.bmp";
BMAP* bmColBlue1   = "bitmaps/button_col_blue1.bmp";
BMAP* bmColBlue2   = "bitmaps/button_col_blue2.bmp";
BMAP* bmColYellow1 = "bitmaps/button_col_yellow1.bmp";
BMAP* bmColYellow2 = "bitmaps/button_col_yellow2.bmp";
BMAP* bmColWhite1  = "bitmaps/button_col_white1.bmp";
BMAP* bmColWhite2  = "bitmaps/button_col_white2.bmp";
BMAP* bmColBlack1  = "bitmaps/button_col_black1.bmp";
BMAP* bmColBlack2  = "bitmaps/button_col_black2.bmp";
BMAP* bmColGrey1   = "bitmaps/button_col_grey1.bmp";
BMAP* bmColGrey2   = "bitmaps/button_col_grey2.bmp";

BMAP* bmTPic1x1_1 = "bitmaps/token_1x1_pic1.bmp";
BMAP* bmTPic1x1_2 = "bitmaps/token_1x1_pic2.bmp";
BMAP* bmTPic2x1_1 = "bitmaps/token_2x1_pic1.bmp";
BMAP* bmTPic2x1_2 = "bitmaps/token_2x1_pic2.bmp";
BMAP* bmTPic2x2_1 = "bitmaps/token_2x2_pic1.bmp";
BMAP* bmTPic2x2_2 = "bitmaps/token_2x2_pic2.bmp";
BMAP* bmTPic2x8_1 = "bitmaps/token_2x8_pic1.bmp";
BMAP* bmTPic2x8_2 = "bitmaps/token_2x8_pic2.bmp";

BMAP* bmTCreate_1 = "bitmaps/create_token1.bmp";
BMAP* bmTDelete_1 = "bitmaps/delete_token1.bmp";

BMAP* bmTLoad  = "bitmaps/load_button.bmp";
BMAP* bmTSave  = "bitmaps/save_button.bmp";
BMAP* bmTReset = "bitmaps/reset_button.bmp";

BMAP* bmTHelp 		  = "bitmaps/help_button.bmp";
BMAP* bmTScreenshot = "bitmaps/screen_button.bmp";
BMAP* bmTQuit 		  = "bitmaps/quit_button.bmp";

BMAP* bmBumpmap = "bitmaps/lmBaseplate.bmp";

BMAP* bmGuiBg1 = "bitmaps/bg_gui_1.bmp";
BMAP* bmGuiBg2 = "bitmaps/bg_gui_2.bmp";
BMAP* bmGuiBg3 = "bitmaps/bg_gui_3.bmp";

BMAP* bmHelpScreen = "bitmaps/helpscreen.bmp";
BMAP* bmLoadScreen = "bitmaps/loadscreen.bmp";
BMAP* bmSaveScreen = "bitmaps/savescreen.bmp";
BMAP* bmOkScreen   = "bitmaps/okscreen.bmp";
BMAP* bmErrorScreen = "bitmaps/errorscreen.bmp";


// ------------------------------------------------------------------
// Struct holding the info of each game token
// ------------------------------------------------------------------
typedef struct 
{ 
  ENTITY*   Token;
  var 		iNumberOfTracePoints;
  VECTOR 	vTracepoints[8];
  VECTOR 	vRotatedTracepoints[8];
  var    	bIsRotateable;
    
} S_TOKENINFO;

// ------------------------------------------------------------------
// Global variables
// ------------------------------------------------------------------
S_TOKENINFO gl_TokenInfo[MAX_TOKENS];
var     		gl_MovObj;
var         gl_ColorSelected;
var         gl_TypeSelected;
var         gl_ObjectsInUse;
var         gl_ObjectsMaxUse;
STRING* 		gl_mouseoverhelpString = "#200";
STRING* 		gl_inputLineString = "#200";
STRING* 		gl_errormsgString = "#200";
STRING* 		gl_okmsgString = "#200";

// ------------------------------------------------------------------
// Panel definitions
// ------------------------------------------------------------------ 
PANEL* Gui_Colors_Pan = // Color selection panel
{
  pos_x = 567; pos_y = 5;
  
  bmap = bmGuiBg1;
  size_x = 290;
  size_y = 42;
  
  button_radio(  9,6, bmColRed1,   bmColRed2,   bmColRed2,   onclickColorButton,NULL,onMouseOverGui);
  button_radio( 49,6, bmColGreen1, bmColGreen2, bmColGreen2, onclickColorButton,NULL,onMouseOverGui);
  button_radio( 89,6, bmColBlue1,  bmColBlue2,  bmColBlue2,  onclickColorButton,NULL,onMouseOverGui);
  button_radio(129,6, bmColYellow1,bmColYellow2,bmColYellow2,onclickColorButton,NULL,onMouseOverGui);
  button_radio(169,6, bmColWhite1, bmColWhite2, bmColWhite2, onclickColorButton,NULL,onMouseOverGui);
  button_radio(209,6, bmColBlack1, bmColBlack2, bmColBlack2, onclickColorButton,NULL,onMouseOverGui);
  button_radio(249,6, bmColGrey1,  bmColGrey2,  bmColGrey2,  onclickColorButton,NULL,onMouseOverGui);

  flags =  VISIBLE | OVERLAY;
}

PANEL* Gui_Tokens_Pan = // Token type selection panel
{
  pos_x = 167; pos_y = 5;
  
  bmap = bmGuiBg1;
  size_x = 290;
  size_y = 42;
  
  button_radio( 18,5, bmTPic1x1_2, bmTPic1x1_1, bmTPic1x1_1, onclickTokenButton,NULL,onMouseOverGui);
  button_radio( 86,5, bmTPic2x1_2, bmTPic2x1_1, bmTPic2x1_1, onclickTokenButton,NULL,onMouseOverGui);
  button_radio(154,5, bmTPic2x2_2, bmTPic2x2_1, bmTPic2x2_1, onclickTokenButton,NULL,onMouseOverGui);
  button_radio(222,5, bmTPic2x8_2, bmTPic2x8_1, bmTPic2x8_1, onclickTokenButton,NULL,onMouseOverGui);
  
  flags =  VISIBLE | OVERLAY;
}

PANEL* Gui_CreateDelete_Pan = // Add-remove token panel
{
  pos_x = 467; pos_y = 5;
  
  bmap = bmGuiBg2;
  size_x = 90;
  size_y = 42;
  
  button(  6,5, bmTCreate_1, bmTCreate_1, bmTCreate_1, onclickCreateButton,NULL,onMouseOverGui);
  button( 52,5, bmTDelete_1, bmTDelete_1, bmTDelete_1, onclickDeleteButton,NULL,onMouseOverGui);
    
  flags = VISIBLE | OVERLAY;
}

PANEL* Gui_ShowTokenNumbers_Pan = // Show 'used of max' tokens numbers
{
  pos_x = 867; pos_y = 721;
  
  bmap = bmGuiBg3;
  size_x = 147;
  size_y =  42;
  
  red   = 200;
  green = 200;
  blue  = 200; 
  
  digits( 10,9,"%3.0f","Courier#24b",1,gl_ObjectsInUse);
  digits( 65,9,"/","Courier#22b",1,0);
  digits( 95,9,"%3.0f","Courier#24b",1,gl_ObjectsMaxUse);
  
  flags = VISIBLE | OVERLAY;
}

PANEL* Gui_LoadSave_Pan = // Load-Save panel
{
  pos_x = 867; pos_y = 5;
  
  bmap = bmGuiBg3;
  size_x = 147;
  size_y =  42;
  
  button( 10,5, bmTSave, bmTSave, bmTSave, onclickSaveButton, NULL, onMouseOverGui);
  button( 57,5, bmTLoad, bmTLoad, bmTLoad, onclickLoadButton, NULL, onMouseOverGui);
  button(104,5, bmTReset, bmTReset, bmTReset, onclickResetButton, NULL, onMouseOverGui);
  
  flags = VISIBLE | OVERLAY;
}

PANEL* Gui_QuitHelp_Pan = // Help-quit-Screenshot panel
{
  pos_x = 10; pos_y = 5;
  
  bmap = bmGuiBg3;
  size_x = 147;
  size_y =  42;
  
  button( 10,5, bmTHelp, bmTHelp, bmTHelp, onclickHelpButton, NULL, onMouseOverGui);
  button( 57,5, bmTScreenshot, bmTScreenshot, bmTScreenshot, onclickScreenshotButton, NULL, onMouseOverGui);
  button(104,5, bmTQuit, bmTQuit, bmTQuit, onclickQuitButton, NULL, onMouseOverGui);
  
  flags = VISIBLE | OVERLAY;
}

PANEL* Gui_Helpscreen_Pan = // The help screen
{
  pos_x = 312; pos_y = 68;
  
  bmap   = bmHelpScreen;
  size_x = 400;
  size_y = 600;
}

TEXT* mouseoverhelpText =	// The help text to be shown on 'mouse-over' button
{
  layer = 2;
  pos_x = 0;
  pos_y = 0;
  size_y = 40;
  offset_y = 0;
  font   = "Arial#16";
  string (gl_mouseoverhelpString);
  flags = CENTER_X | TRANSPARENT;
}

PANEL* saveWindow_pan = // The save build input panel 
{
	bmap  = bmSaveScreen;
	layer = 2;
	flags = TRANSLUCENT; // Initially, not visible
	size_x = 400;
	size_y = 185;
	pos_x = 312; 
	pos_y = 290;
	alpha = 100;
}

PANEL* loadWindow_pan = // The load build input panel
{
	bmap  = bmLoadScreen;
	layer = 2;
	flags = TRANSLUCENT; // Initially, not visible
	size_x = 400;
	size_y = 185;
	pos_x = 312; 
	pos_y = 290;
	alpha = 100;
}

TEXT* inputText = // The save build input text. Not integrated in a panel, as inkey needs a text for showing the cursor
{
	layer = 2;
	flags = TRANSLUCENT; // Initially, not visible
	font ="Arial#20";
	pos_x = 344;
	pos_y = 432;
	alpha = 100;
	string = gl_inputLineString;
	red   = 0;
   green = 0;
   blue  = 0; 
}

PANEL* errorWindow_pan = // The error output panel
{
	bmap  = bmErrorScreen;
	layer = 2;
	size_x = 400;
	size_y = 185;
	pos_x = 312; 
	pos_y = 290;
	alpha = 100;
	red   = 0;
   green = 0;
   blue  = 0; 
	digits(145,100,"%s","Arial#26",1,gl_errormsgString);            
}

PANEL* okWindow_pan = // The ok output panel
{
	bmap  = bmOkScreen;
	layer = 2;
	size_x = 400;
	size_y = 185;
	pos_x = 312; 
	pos_y = 290;
	alpha = 100;
	red   = 0;
   green = 0;
   blue  = 0; 
	digits(145,100,"%s","Arial#26",1,gl_okmsgString);            
}

// ------------------------------------------------------------------
// Material definition for bumpmaps of the outer playfield boxes
// ------------------------------------------------------------------
MATERIAL* matBump = // Definition copied from the 3dgs helpfile
{
   skin2 = bmBumpmap; 
   scale1 = 0.2;

   effect = "  texture entSkin1;
               texture mtlSkin2;
               dword   mtlSkill1;

               // default Technique
               technique bump_dot3
               {
                  pass P0
                  {
                      Texture[0] = <mtlSkin2>;
                      Texture[1] = <entSkin1>;
                      TextureFactor = <mtlSkill1>;
  
                      ColorArg1[0] = Texture; // stage 0 = bumpmap
                      ColorOp[0] = DotProduct3;
                      ColorArg2[0] = TFactor;
  
                      ColorArg1[1] = Texture; // stage 1 - skin texture
                      ColorOp[1] = AddSigned;
                      ColorArg2[1] = Current;
  
                      ColorArg1[2] = Diffuse; // stage 2 - lighting
                      ColorOp[2] = Modulate2x;
                      ColorArg2[2] = Current;
                  }
               }

               technique fallback
               {
                  pass P0
                  {
                     Texture[0] = <entSkin1>;
                     ColorArg1[0] = Texture; // stage 0 = skin texture
                     ColorOp[0] = Modulate2x;
                     ColorArg2[0] = Diffuse;
                  }
               } "; 
}

// ------------------------------------------------------------------
// Sounds, downloaded from http://www.partnersinrhyme.com. Great site
// ------------------------------------------------------------------ 
SOUND*  snd_Click = "data/CLICK10B.wav";
SOUND*  snd_Error = "data/BADUMM.wav";


// ----------------------------------------------------------------------------------------------
function round(x) // General helper function
// ----------------------------------------------------------------------------------------------
{
	return(integer(x+0.5*x/abs(x))); 
}

// ----------------------------------------------------------------------------------------------
function floor(x) // General helper function
// ----------------------------------------------------------------------------------------------
{
	return(integer(x)); 
}

// ----------------------------------------------------------------------------------------------
function matBump_startup (void) // Initialisation of bumpmap material at startup
// ----------------------------------------------------------------------------------------------
{
   vec_set(matBump.ambient_blue,  mat_model.ambient_blue);
   vec_set(matBump.diffuse_blue,  mat_model.diffuse_blue);
   vec_set(matBump.specular_blue, mat_model.specular_blue);
   matBump.power = mat_model.power;

   bmap_to_normals(matBump.skin2,2);

   matBump.skill1 = pixel_for_vec(vector(200,200,200),100,8888);
}

// ----------------------------------------------------------------------------------------------
function initializeData_startup(void) // General initialisations at startup
// ----------------------------------------------------------------------------------------------
{
	resetCameraposition();
	
	camera.aspect = 1024/768; // Using fixed resolution here

	gl_MovObj = NONE;
	gl_ColorSelected = COLOR_RED;
	gl_TypeSelected  = TOKEN_2X2;
	gl_ObjectsInUse  = 0;
	gl_ObjectsMaxUse = MAX_TOKENS;
	
	var i;
	for (i=0;i<MAX_TOKENS;i++)
	{
		gl_TokenInfo[i].Token = NULL;
	}
	
	button_state(Gui_Colors_Pan,gl_ColorSelected,ON); 
	button_state(Gui_Tokens_Pan,gl_TypeSelected,ON);
	
	mouse_range = 5000; // For selection of tokens at larger distances	
}

// ----------------------------------------------------------------------------------------------
function moveCamera_startup(void) // Handling of the camera movement
// ----------------------------------------------------------------------------------------------
{
   while(1)
   {
   	if (mouse_right==1)
   	{
   		var angle = 0;
   		angle = (-mouse_force.x*time_step*10);
   		
   		// distance cam to 0/0
   		VECTOR vTemp;
   		vec_set(vTemp,camera.x);
   		vec_rotate(vTemp,vector(angle,0,0));
   		vec_set(camera.x,vTemp);
   		camera.pan += angle; 
	  	}
	  	
  		if (key_cuu && key_shift) { camera.z += 2*time_step; } 		 // move cam up
  		else if (key_cud && key_shift) { camera.z -= 2*time_step; }  // move cam down
   	
   	else if (key_cul) { camera.pan  += time_step; } // rotate left/right
  		else if (key_cur) { camera.pan  -= time_step; }	
  		
  		else if (key_cuu) { camera.tilt += time_step; } // rotate up/down
  		else if (key_cud) { camera.tilt -= time_step; }
  		
  		else if (camera.tilt > 80)  { camera.tilt=80;  } // checking for some limits
  		else if (camera.tilt < -80) { camera.tilt=-80; }
		
   	wait(1);
   }
}

// ----------------------------------------------------------------------------------------------
function moveMouse_startup(void) // Handling the mouse 
// ----------------------------------------------------------------------------------------------
{
   mouse_mode = 1;
   updateMouseCursor(); // Initially, selecting the right mouse cursor
   
   while(1)
   {
   	vec_set(mouse_pos, mouse_cursor);
   	  	
   	if (mickey.z!=0)
   	{
   		if (gl_MovObj==NONE) // Moving the camera around
   		{
				VECTOR vDir;
   			vec_set(vDir,vector(1,0,0));
   			vec_rotate(vDir,camera.pan);
   			vec_scale(vDir,mickey.z*0.5);
   		
   			VECTOR vTemp;
   			vec_set(vTemp,camera.x);
   			vec_add(vTemp,vDir);
   			if (vTemp.z>15) vec_set(camera.x,vTemp);
   		}
   		else // Moving a token. Mouse wheel rotates it left right
   		{
   			if (mickey.z>0) rotateTokenRight();
   			if (mickey.z<0) rotateTokenLeft();  
			}
   	}
   	
   	wait(1);
   }
}

// ----------------------------------------------------------------------------------------------
function mouseoverHelp_startup(void) // Showing mouseover-help, if activated
// ----------------------------------------------------------------------------------------------
{
	while(1)
	{
		if (is(mouseoverhelpText, VISIBLE))
		{
			if(mouseoverhelpText.alpha > 0)
      	{
       		mouseoverhelpText.alpha -= 1.5*time_step;
      	}
      	else reset(mouseoverhelpText, VISIBLE);
		}
	
		wait(1);
	}
}

// ----------------------------------------------------------------------------------------------
function errormsgPan_startup(void) // Showing the error window, if activated
// ----------------------------------------------------------------------------------------------
{
	while(1)
	{
		if (is(errorWindow_pan, VISIBLE))
		{
			if(errorWindow_pan.alpha > 0)
      	{
       		errorWindow_pan.alpha -= 1.5*time_step;
      	}
      	else reset(errorWindow_pan, VISIBLE);
		}
	
		wait(1);
	}
}

// ----------------------------------------------------------------------------------------------
function okmsgPan_startup(void) // Showing the ok window, if activated
// ----------------------------------------------------------------------------------------------
{
	while(1)
	{
		if (is(okWindow_pan, VISIBLE))
		{
			if(okWindow_pan.alpha > 0)
      	{
       		okWindow_pan.alpha -= 1.5*time_step;
      	}
      	else reset(okWindow_pan, VISIBLE);
		}
	
		wait(1);
	}
}

// ----------------------------------------------------------------------------------------------
function showErrorMessage(STRING* errortext) // Showing the error message, if activated
// ----------------------------------------------------------------------------------------------
{
	snd_play(snd_Error,50,0);
	
	str_cpy(gl_errormsgString,errortext);
	errorWindow_pan.alpha = 100;
	set(errorWindow_pan,VISIBLE);
}

// ----------------------------------------------------------------------------------------------
function showOkMessage(STRING* oktext) // Showing the ok message, if activated
// ----------------------------------------------------------------------------------------------
{
	snd_play(snd_Error,50,0);
	
	str_cpy(gl_okmsgString,oktext);
	okWindow_pan.alpha = 100;
	set(okWindow_pan,VISIBLE);
}

// ----------------------------------------------------------------------------------------------
function deleteContent(void) // Reset token infos
// ----------------------------------------------------------------------------------------------
{
	var i;
	for (i=0;i<MAX_TOKENS;i++)
	{
		if(gl_TokenInfo[i].Token != NULL)
		{
			ent_remove(gl_TokenInfo[i].Token);
			gl_TokenInfo[i].Token = NULL;
		} 
	}
	
	gl_MovObj = NONE;
	gl_ObjectsInUse  = 0;
}

// ----------------------------------------------------------------------------------------------
function resetCameraposition(void) // Go back to initial cam 
// ----------------------------------------------------------------------------------------------
{
	camera.x = -0;
	camera.y = -540;
	camera.z =  190;
	
	camera.roll =   0;
	camera.pan  =  90;
	camera.tilt = -20;
}

// ----------------------------------------------------------------------------------------------
function setTokenColor(index,color)
// ----------------------------------------------------------------------------------------------
{
	gl_TokenInfo[index].Token.iObjectColor = color;
	
	switch(color)
	{
		case COLOR_RED:    ent_setskin(gl_TokenInfo[index].Token,bmRed,1);break;
		case COLOR_GREEN:  ent_setskin(gl_TokenInfo[index].Token,bmGreen,1);break;	
		case COLOR_BLUE:   ent_setskin(gl_TokenInfo[index].Token,bmBlue,1);break;	
		case COLOR_YELLOW: ent_setskin(gl_TokenInfo[index].Token,bmYellow,1);break;	
		case COLOR_WHITE:  ent_setskin(gl_TokenInfo[index].Token,bmWhite,1);break;	
		case COLOR_BLACK:  ent_setskin(gl_TokenInfo[index].Token,bmBlack,1);break;
		case COLOR_GREY:   ent_setskin(gl_TokenInfo[index].Token,bmGrey,1);break;	
	}
}

// ----------------------------------------------------------------------------------------------
function onshowHelp(void) // Show or hide the help screen
// ----------------------------------------------------------------------------------------------
{
	if (is(Gui_Helpscreen_Pan,VISIBLE))
	{
		reset(Gui_Helpscreen_Pan,VISIBLE);
	}
	else
	{
		set(Gui_Helpscreen_Pan,VISIBLE);
	}
}

// ----------------------------------------------------------------------------------------------
function setmouseoverHelp(x,y,STRING* message) // Assign an mouseover help message and activate
// ----------------------------------------------------------------------------------------------
{
	str_cpy(gl_mouseoverhelpString,message);
	set(mouseoverhelpText,VISIBLE);
	mouseoverhelpText.alpha = 100;
	mouseoverhelpText.pos_x = x;
	mouseoverhelpText.pos_y = y; 
}

// ----------------------------------------------------------------------------------------------
function onMouseOverGui(button_number,PANEL* panel) // Select the right mouseover help text
// ----------------------------------------------------------------------------------------------
{
	if (panel==Gui_Tokens_Pan) 
	{
		switch (button_number)
		{
			case 1: setmouseoverHelp(312,50,"New token will have size 1x1"); return; break;
			case 2: setmouseoverHelp(312,50,"New token will have size 2x1"); return; break;
			case 3: setmouseoverHelp(312,50,"New token will have size 2x2"); return; break;
			case 4: setmouseoverHelp(312,50,"New token will have size 2x8"); return; break;
		}
	}
	else if (panel==Gui_CreateDelete_Pan) 
	{
		switch (button_number)
		{
			case 1: setmouseoverHelp(512,50,"Add a new token");       return; break;
			case 2: setmouseoverHelp(512,50,"Delete selected token"); return; break;
		}
	}
	else if (panel==Gui_Colors_Pan) 
	{
		switch (button_number)
		{
			case 1: setmouseoverHelp(712,50,"New token will have red color");   return; break;
			case 2: setmouseoverHelp(712,50,"New token will have green color"); return; break;
			case 3: setmouseoverHelp(712,50,"New token will have blue color");  return; break;
			case 4: setmouseoverHelp(712,50,"New token will have yellow color");return; break;
			case 5: setmouseoverHelp(712,50,"New token will have white color"); return; break;
			case 6: setmouseoverHelp(712,50,"New token will have black color"); return; break;
			case 7: setmouseoverHelp(712,50,"New token will have grey color");  return; break;		
		}
	}
	else if (panel==Gui_LoadSave_Pan) 
	{
		switch (button_number)
		{
			case 1: setmouseoverHelp(940,50,"Save build to disk");  return; break;
			case 2: setmouseoverHelp(940,50,"Load build from disk");return; break;
			case 3: setmouseoverHelp(940,50,"Delete all tokens");   return; break;
		}
	}
	else if (panel==Gui_QuitHelp_Pan) 
	{
		switch (button_number)
		{
			case 1: setmouseoverHelp( 88,50,"Help (F1)");       return; break;
			case 2: setmouseoverHelp( 88,50,"Save Screenshot"); return; break;
			case 3: setmouseoverHelp( 88,50,"Quit now");        return; break;
		}
	}
}

// ----------------------------------------------------------------------------------------------
function onclickResetButton(button_number,PANEL* panel) // Delete content, show message
// ----------------------------------------------------------------------------------------------
{
  snd_play(snd_Click,50,0);
  deleteContent();
  showOkMessage("All data reset.");
}

// ----------------------------------------------------------------------------------------------
function onclickSaveButton(button_number,PANEL* panel) // Open save window
// ----------------------------------------------------------------------------------------------
{
  snd_play(snd_Click,50,0);
  if (is(saveWindow_pan,VISIBLE)) return;
  
  str_cpy(gl_inputLineString,"#25");
  
  set(saveWindow_pan,VISIBLE);
  set(inputText, VISIBLE); 
  inkey (gl_inputLineString);
  
  reset(saveWindow_pan,VISIBLE);
  reset(inputText, VISIBLE);
  
  if (result == 13)
  {
  		str_cat(gl_inputLineString,".dat");
  		
  		var filehandle;
		filehandle = file_open_write(gl_inputLineString);
		
		if (!filehandle)
		 {
			showErrorMessage("Could not create save file.");
			reset(saveWindow_pan,VISIBLE);
  			reset(inputText, VISIBLE);   
   		return;
		}
		else
		{
			file_var_write(filehandle,gl_ObjectsInUse);
			
			var i;
			for (i=0;i<gl_ObjectsInUse;i++)
			{
				file_var_write(filehandle,gl_TokenInfo[i].Token.iObjectType);
				file_var_write(filehandle,gl_TokenInfo[i].Token.iObjectColor);
				file_var_write(filehandle,gl_TokenInfo[i].Token.pan*1000); // *1000 to compensate rounding
				file_var_write(filehandle,gl_TokenInfo[i].Token.x*1000);
				file_var_write(filehandle,gl_TokenInfo[i].Token.y*1000);
				file_var_write(filehandle,gl_TokenInfo[i].Token.z*1000);
			}
			
			file_close(filehandle);
		} 
  		
  		showOkMessage("Data saved."); 
  } 
}

// ----------------------------------------------------------------------------------------------
function onclickLoadButton(button_number,PANEL* panel) // Open load window. No file listing there
// ----------------------------------------------------------------------------------------------
{
  snd_play(snd_Click,50,0);
  if (is(loadWindow_pan,VISIBLE)) return;
  
  str_cpy(gl_inputLineString,"#25");
  
  set(loadWindow_pan,VISIBLE);
  set(inputText, VISIBLE); 
  inkey (gl_inputLineString);
  
  reset(loadWindow_pan,VISIBLE);
  reset(inputText, VISIBLE);
  
  if (result == 13)
  {
		str_cat(gl_inputLineString,".dat");
		
		var filehandle;
		filehandle = file_open_read(gl_inputLineString);
		
		if (!filehandle)
		{
			showErrorMessage("Could not open load file.");
			reset(loadWindow_pan,VISIBLE);
  			reset(inputText, VISIBLE);   
   		return;
		}
		else
		{
			deleteContent(); 
			var number = file_var_read(filehandle);
			
			var i;
			for (i=0;i<number;i++)
			{
				gl_TypeSelected  = file_var_read(filehandle);
				gl_ColorSelected = file_var_read(filehandle);
				gl_MovObj = NONE; 
				loadModel();
				
				if (gl_TokenInfo[i].Token==NULL) 
				{
					showErrorMessage("Error reading save file.");
					file_close(filehandle);
					deleteContent();
					reset(loadWindow_pan,VISIBLE);
					reset(inputText, VISIBLE); 
					return;
				}
				else
				{
					gl_TokenInfo[i].Token.pan = file_var_read(filehandle)/1000; // has been saved with *1000
					gl_TokenInfo[i].Token.x = file_var_read(filehandle)/1000;
					gl_TokenInfo[i].Token.y = file_var_read(filehandle)/1000;
					gl_TokenInfo[i].Token.z = file_var_read(filehandle)/1000;
				
					gl_TokenInfo[i].Token.bMovemodus = false;
				}
			}
			
			file_close(filehandle);
			gl_MovObj = NONE;
			
			showOkMessage("Data loaded.");  
		}
	} 
}

// ----------------------------------------------------------------------------------------------
function onclickTokenButton(button_number,PANEL* panel) // Change token type on next creation
// ----------------------------------------------------------------------------------------------
{
  snd_play(snd_Click,50,0); 
  gl_TypeSelected = button_number;
}

// ----------------------------------------------------------------------------------------------
function onclickColorButton(button_number,PANEL* panel) // Change token color on next creation
// ----------------------------------------------------------------------------------------------
{
  snd_play(snd_Click,50,0);
  gl_ColorSelected = button_number;
}

// ----------------------------------------------------------------------------------------------
function onclickCreateButton(button_number,PANEL* panel) // Create a new token
// ----------------------------------------------------------------------------------------------
{
  snd_play(snd_Click,50,0);
  loadModel();
}

// ----------------------------------------------------------------------------------------------
function onclickDeleteButton(button_number,PANEL* panel) // Delete the currently selected token
// ----------------------------------------------------------------------------------------------
{
  snd_play(snd_Click,50,0);
  deleteActiveModel();  
}

// ----------------------------------------------------------------------------------------------
function onclickHelpButton(button_number,PANEL* panel) // Show help (F1 also)
// ----------------------------------------------------------------------------------------------
{
	snd_play(snd_Click,50,0);
	onshowHelp();
}

// ----------------------------------------------------------------------------------------------
function onclickScreenshotButton(button_number,PANEL* panel) // Invoke Screeshot creation
// ----------------------------------------------------------------------------------------------
{
	snd_play(snd_Click,50,0);
	saveScreenshot();
	wait(1);
	showOkMessage("Screenshot saved.");
}

// ----------------------------------------------------------------------------------------------
function onclickQuitButton(button_number,PANEL* panel) // End the programm. No "are you sure" here
// ----------------------------------------------------------------------------------------------
{
	snd_play(snd_Click,50,0);
	sys_exit("Good bye...");
}

// ----------------------------------------------------------------------------------------------
function updateMouseCursor(void) // Select the right cursor for the current action
// ----------------------------------------------------------------------------------------------
{
	if (gl_MovObj != NONE)
  	{
  		mouse_pointer=1; // Hand
  	}
  	else
  	{
		mouse_pointer=2; // Arrow
	}
}

// Each token has got some tracepoints that are used to check if it can be placed at a certain
// position. Here, the defined tracepoints are recalculated by the objects pan angle.
// ----------------------------------------------------------------------------------------------
function calculateRotatedTracepoints(var iIndex)
// ----------------------------------------------------------------------------------------------
{
	var i;
	for (i=0;i<gl_TokenInfo[iIndex].iNumberOfTracePoints;i++)
	{
		VECTOR vTmp1, vTmp2;
		vec_set(gl_TokenInfo[iIndex].vRotatedTracepoints[i],gl_TokenInfo[iIndex].vTracepoints[i]);
		vec_rotate(gl_TokenInfo[iIndex].vRotatedTracepoints[i],vector(gl_TokenInfo[iIndex].Token.pan,0,0));
	}
}

// Check if tracepoints show that the token is outside the placing area
// ----------------------------------------------------------------------------------------------
function violateBoundaryCondition(var iIndex) 
// ----------------------------------------------------------------------------------------------
{
	var i;
	for (i=0;i<gl_TokenInfo[iIndex].iNumberOfTracePoints;i++)
	{
		VECTOR vTmp1, vTmp2;
		vec_set(vTmp1,gl_TokenInfo[iIndex].vRotatedTracepoints[i]);
		
		vec_set(vTmp2,gl_TokenInfo[iIndex].Token.x);
		vec_add(vTmp2,vTmp1);
			
		if (abs(vTmp2.x/16)>12) return(true); 
		if (abs(vTmp2.y/16)>12) return(true);
	}
	return(false);
}

// ----------------------------------------------------------------------------------------------
function clickGlobal(void) // Catches "global" click events. Here, to deselect a token
// ----------------------------------------------------------------------------------------------
{
	if (gl_MovObj != NONE)
  	{
  		gl_TokenInfo[gl_MovObj].Token.bMovemodus = false;
  		gl_MovObj = NONE;
  		updateMouseCursor();
  	}  	
}

// ----------------------------------------------------------------------------------------------
function clickToken(void) // Click on a token happened. 
// ----------------------------------------------------------------------------------------------
{
	if ((event_type == EVENT_CLICK))
  	{
  		if (gl_MovObj != NONE) // A token was selected. Unselect now
  		{
  			gl_TokenInfo[gl_MovObj].Token.bMovemodus = false;
  			gl_MovObj = NONE;
  		}  		
  		else
  		{
  			// Check if the token is moveable
  			var bStoneUpThere = false;
  			var i;
			for (i=0;i<gl_TokenInfo[me.iObjectIndex].iNumberOfTracePoints;i++)
			{
				VECTOR vTmp1, vTmp2;
				vec_set(vTmp1,gl_TokenInfo[me.iObjectIndex].vRotatedTracepoints[i]);
									
				vec_set(vTmp2,me.x);
				vec_add(vTmp2,vTmp1);
				vec_set(vTmp1,vTmp2);
				
				vTmp1.z+=  8;
				vTmp2.z+= 24;
				
				var ret;
				ret = c_trace(vTmp1,vTmp2,IGNORE_ME|IGNORE_PASSABLE|IGNORE_CONTENT);
				if (ret!=0) bStoneUpThere=true;				
			}
  			
  			if(bStoneUpThere==false)
  			{
				me.bMovemodus = true;
  				gl_MovObj = me.iObjectIndex;	
  			}  			
  		}
  		updateMouseCursor();
	}
}

// ----------------------------------------------------------------------------------------------
function moveToken(void) // Draw bounding lines to the object. Draw "ropes" that lift the token and move it
// ----------------------------------------------------------------------------------------------
{
	wait(1);
	c_setminmax(me);
	wait(1);
	
	set(me,ENABLE_CLICK);
	me.emask |= ENABLE_CLICK;
   me.event = clickToken;
   
   reset(me,SHADOW);
               
   VECTOR color;
	
	switch(me.iObjectColor)
	{
		case COLOR_RED:    vec_set(color,vector( 50, 50,255));break;
		case COLOR_GREEN:  vec_set(color,vector( 50,255, 50));break;	
		case COLOR_BLUE:   vec_set(color,vector(255, 50, 50));break;	
		case COLOR_YELLOW: vec_set(color,vector( 50,255,255));break;	
		case COLOR_WHITE:  vec_set(color,vector(200,200,200));break;	
		case COLOR_BLACK:  vec_set(color,vector( 50, 50, 50));break;
		case COLOR_GREY:   vec_set(color,vector( 50, 50, 50));break;	
	}
	
	while(1)
	{
		VECTOR vP[8];
		var i;
		
		for (i=0;i<8;i++) vec_for_mesh (vP[i],me,i+1);
		for (i=0;i<8;i++) vec_rotate(vP[i],me.pan);
		for (i=0;i<8;i++) vec_add(vP[i],me.x);	
		
		draw_line3d(vP[0],NULL,80); // This looks as if the "bounding box mode" would be active by F11
		draw_line3d(vP[0],color,80);
		draw_line3d(vP[1],color,80);
		draw_line3d(vP[2],color,80); 
		draw_line3d(vP[3],color,80);
		draw_line3d(vP[0],color,80);
		draw_line3d(vP[4],color,80);
		draw_line3d(vP[5],color,80);
		draw_line3d(vP[6],color,80);
		draw_line3d(vP[7],color,80);
		draw_line3d(vP[4],color,80);
		draw_line3d(vP[1],NULL,80);
		draw_line3d(vP[5],color,80);
		draw_line3d(vP[2],NULL,80);
		draw_line3d(vP[6],color,80);
		draw_line3d(vP[3],NULL,80);
		draw_line3d(vP[7],color,80);
		
		
		if (me.bMovemodus == true)
		{
			VECTOR vec1;
			
			vec_set(vec1,mouse_dir3d);
			vec_normalize(vec1,5000);
			vec_add(vec1,camera.x);
			
			var ret = c_trace(vector(camera.x,camera.y,camera.z), vec1, IGNORE_ME|IGNORE_PASSABLE|IGNORE_CONTENT|IGNORE_MODELS);
			
		
			VECTOR vNewPos;
			var bestRet=1000;
			var bOutOfBound=false;
				
			if (ret!=0) 
			{	
				vNewPos.x = 16*(round(target.x/16));
				vNewPos.y = 16*(round(target.y/16));
				vNewPos.z = 1000; // to be adapted below				
			}
			else
			{
				vNewPos.x = me.x;
				vNewPos.y = me.y;
				vNewPos.z = 1000; // to be adapted below
			}
			
			var i;
			for (i=0;i<gl_TokenInfo[gl_MovObj].iNumberOfTracePoints;i++)
			{
				VECTOR vTmp1, vTmp2;
				vec_set(vTmp1,gl_TokenInfo[gl_MovObj].vRotatedTracepoints[i]);
				
				vec_set(vTmp2,vNewPos);
				vec_add(vTmp2,vTmp1);
				vec_set(vTmp1,vTmp2);
				
				vTmp1.z = 1000;
				vTmp2.z = -5;
				
				draw_line3d(vTmp1,NULL,100); 
				draw_line3d(vTmp2,vector(0,255,255),80);
				
				var ret;
				ret = c_trace(vTmp1,vTmp2,IGNORE_ME|IGNORE_PASSABLE|IGNORE_CONTENT);
				if (ret<bestRet) { bestRet = ret; }	
			}
				
			vNewPos.z -= (bestRet-0.1);
			
			VECTOR vOldPos;
			vec_set(vOldPos,me.x);
			vec_set(me.x, vNewPos);
			if (violateBoundaryCondition(gl_MovObj)==true) 
			{
				vec_set(me.x, vOldPos);
			}
		}
		wait(1);
	}
}

// ----------------------------------------------------------------------------------------------
function rotateTokenLeft(void) // Rotation is done by the user with the mouse wheel. 90 steps
// ----------------------------------------------------------------------------------------------
{
	if (gl_MovObj==NONE) return;
	if (gl_TokenInfo[gl_MovObj].bIsRotateable==false) return;
	gl_TokenInfo[gl_MovObj].Token.pan += 90;
	calculateRotatedTracepoints(gl_MovObj);
	
	if (violateBoundaryCondition(gl_MovObj)==false) return;
	
	gl_TokenInfo[gl_MovObj].Token.pan -= 90;
	calculateRotatedTracepoints(gl_MovObj);
}

// ----------------------------------------------------------------------------------------------
function rotateTokenRight(void) // Rotation is done by the user with the mouse wheel. 90 steps
// ----------------------------------------------------------------------------------------------
{
	if (gl_MovObj==NONE) return;
	if (gl_TokenInfo[gl_MovObj].bIsRotateable==false) return;
	gl_TokenInfo[gl_MovObj].Token.pan -= 90;
	calculateRotatedTracepoints(gl_MovObj);
	
	if (violateBoundaryCondition(gl_MovObj)==false) return;
	
	gl_TokenInfo[gl_MovObj].Token.pan += 90;
	calculateRotatedTracepoints(gl_MovObj);
}

// ----------------------------------------------------------------------------------------------
function deleteActiveModel(void) // End its existence
// ----------------------------------------------------------------------------------------------
{
	// None there
	if (gl_MovObj==NONE) return;
	if (gl_TokenInfo[gl_MovObj].Token==NULL) return;
	
	ent_remove(gl_TokenInfo[gl_MovObj].Token);
	gl_TokenInfo[gl_MovObj].Token=NULL;
	gl_MovObj=NONE;
	
	gl_ObjectsInUse--;
}

// ----------------------------------------------------------------------------------------------
function loadModel(void) // Check the definition file (*.txt) and load models
// ----------------------------------------------------------------------------------------------
{
	// Already one moving, do not load another
	if (gl_MovObj != NONE) return;
	
	// First, find free slot
	int i;
	for (i=0;i<MAX_TOKENS;i++)
	{
		if (gl_TokenInfo[i].Token==NULL) break;	
	}
	
	// Check for full list
	if (i==MAX_TOKENS) return;
	
	// Load Definition file
	STRING* fileName = "#50";
	switch(gl_TypeSelected)
	{
		case TOKEN_1X1: str_cpy(fileName,"data/tokendata_1x1.txt");break;
		case TOKEN_2X1: str_cpy(fileName,"data/tokendata_2x1.txt");break;
		case TOKEN_2X2: str_cpy(fileName,"data/tokendata_2x2.txt");break;
		case TOKEN_2X8: str_cpy(fileName,"data/tokendata_2x8.txt");break;
		default: return; break;
	} 
	
	var fileHandle;
	STRING* lineData = "#50";
	fileHandle = file_open_game(fileName);
	if (!fileHandle) return;
	
	var ii;
	for (ii=0;ii<6;ii++) file_str_read(fileHandle,lineData); // Header1..6
		
	file_str_read(fileHandle,lineData); // Model name	
	gl_TokenInfo[i].Token = ent_create(lineData,vector(0,0,1000),moveToken);
	gl_TokenInfo[i].Token.iObjectIndex = i;
	gl_TokenInfo[i].Token.bMovemodus	= true;
	gl_TokenInfo[i].Token.iObjectType = gl_TypeSelected;
	gl_TokenInfo[i].bIsRotateable = file_var_read(fileHandle);
	gl_TokenInfo[i].iNumberOfTracePoints = file_var_read(fileHandle);
	gl_MovObj = i;
	
	for (ii=0;ii<gl_TokenInfo[i].iNumberOfTracePoints;ii++)
	{
		gl_TokenInfo[i].vTracepoints[ii].x=file_var_read(fileHandle);
		gl_TokenInfo[i].vTracepoints[ii].y=file_var_read(fileHandle);
		gl_TokenInfo[i].vTracepoints[ii].z=file_var_read(fileHandle);
	}	
	
	file_close(fileHandle); 
	
	calculateRotatedTracepoints(i);
	updateMouseCursor();
	
	ent_cloneskin(gl_TokenInfo[i].Token); // Needed as the color texture is changed during runtime
	setTokenColor(i,gl_ColorSelected);
	
	gl_ObjectsInUse++;
}

// ----------------------------------------------------------------------------------------------
function saveScreenshot(void) // Screen shots are named 'screenshot_xx', where xx is a rising number
// ----------------------------------------------------------------------------------------------
{
	STRING* filename = "screenshot_";
	STRING* dummy1 = "#50";
	STRING* dummy2 = "#50";
	
	var ok = false;
	var counter = 0;
	while (ok==false)
	{
		counter++;
		str_cpy(dummy1,filename);
		str_for_num(dummy2,counter);
		str_cat(dummy1,dummy2);
		str_cat(dummy1,".jpg");
		var handle = file_open_read (dummy1);
		if (handle==0)
		{
			ok = true;
		}
		else
		{
			file_close(handle);
		}	
	}
	
	file_for_screen (filename,counter);
}

// ----------------------------------------------------------------------------------------------
function main(void)
// ----------------------------------------------------------------------------------------------
{
	// If the user's screen has a resolution of 1024x768, we switch
	// to fullscreen mode (windowed, it wouldn't work). Use windowed
	// mode for larger resolutions
	
	VECTOR desktop_size;
	desktop_size.x = sys_metrics(0); // SM_CXSCREEN
	desktop_size.y = sys_metrics(1); // SM_CYSCREEN

   if ((desktop_size.x<=1024)||(desktop_size.y<=768))
   {
		video_mode = 8;  // 1024x768
		video_screen = 1; // fullscreen
	}
	else
	{
		video_mode = 8;  // 1024x768
		video_screen = 2; // windowed
	}
	
	fps_max = 60;	 // is enough
	randomize(); 	 // I do that always
	
	add_folder("models");
	str_cpy(save_dir, "save");
	
	level_load("models/tokenbuilder.wmb");
	wait(3);
	
	vec_set(sky_color,vector(80,0,0));
	
	// Overriding engine key bindings
	on_f2 = NULL;
	on_f3 = NULL;
	on_f4 = NULL;
	on_f5 = NULL;
	on_f6 = NULL;
	on_f12 = NULL;
	on_esc = NULL;
	
	on_del = onclickDeleteButton;
	on_f1  = onshowHelp;
	on_home = resetCameraposition;
	
	on_click = clickGlobal;
	
	while (1)
	{
     	wait (1);
	}
}